/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



#include "CentralWidget/centralwidget.h"
#include "Widget/splittercollapser.h"
#include "yarocksettings.h"

#include "global_actions.h"
//#include "debug.h"

/*
********************************************************************************
*                                                                              *
*    Class CentralWidget                                                       *
*                                                                              *
********************************************************************************
*/
CentralWidget::CentralWidget(
                       QWidget *parent,
                       CentralToolBar     *centralToolBar,
                       MenuView           *menuView,
                       NowPlayingView     *nowPlayingView,
                       PlaylistWidget     *playlistWidget,
                       GraphicViewWidget  *graphicViewWidget                       
                       ) : QWidget(parent)
{
    //! get all widgets
    m_centraltoolbar     = centralToolBar;
    m_menuView           = menuView;
    m_nowplayingview     = nowPlayingView;
    m_playlistwidget     = playlistWidget;
    m_graphicviewwidget  = graphicViewWidget;

    //! Gui setup
    createGui();
  
    //! Show/Hide actions
    QMap<ENUM_ACTION, QAction*> *actions = ACTIONS();
    connect(actions->value(APP_SHOW_PLAYQUEUE),   SIGNAL(triggered()), SLOT(slot_show_playlist()));
    connect(actions->value(APP_SHOW_MENU),        SIGNAL(triggered()), SLOT(slot_show_menu()));
    connect(actions->value(APP_SHOW_NOW_PLAYING), SIGNAL(triggered()), SLOT(slot_show_nowplaying()));
  
    //! Prev/Next View activation
    connect(m_centraltoolbar,SIGNAL(prevNextActivated(QModelIndex,QString)),this, SLOT(slot_prevnext_activated(QModelIndex,QString)));  

    //! Menu activation
    MenuModel *menuModel = static_cast<MenuModel*>(m_menuView->model());
    connect(menuModel,SIGNAL(modelItemActivated(QModelIndex)),this, SLOT(slot_modelitem_activated(QModelIndex)));  

    //! Restore previous state
    restoreState();
}


//! --------- createGui --------------------------------------------------------
void CentralWidget::createGui()
{
    this->setObjectName(QString::fromUtf8("Main widget"));
    this->setAutoFillBackground(false);
  
    QVBoxLayout* centralWidgetLayout = new QVBoxLayout(this);
    centralWidgetLayout->setSpacing(0);
    centralWidgetLayout->setContentsMargins(0, 0, 0, 0);
  
    //! top Frame
    QWidget *topFrame = new QFrame(this);
    topFrame->setMaximumHeight(28);

    QHBoxLayout* topFrameLayout = new QHBoxLayout(topFrame);
    topFrameLayout->setSpacing(0);
    topFrameLayout->setContentsMargins(0, 0, 0, 0);
    topFrameLayout->addWidget(m_centraltoolbar);

    //! main Frame
    QFrame *mainFrame = new QFrame(this);
    mainFrame->setFrameShape(QFrame::StyledPanel);
    mainFrame->setFrameShadow(QFrame::Sunken);
    mainFrame->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    
    QHBoxLayout* mainFrameLayout = new QHBoxLayout(mainFrame);
    mainFrameLayout->setSpacing(0);
    mainFrameLayout->setContentsMargins(0, 0, 0, 0);

    m_viewsSplitter = new CustomSplitter(this);
    m_viewsSplitter->setObjectName(QString::fromUtf8("viewsSplitter"));
  
    //! right pane
    QFrame* right_widget = new QFrame(this);
    right_widget->setFrameShape(QFrame::NoFrame);
    right_widget->setFrameShadow(QFrame::Plain);
    right_widget->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Expanding );

    QVBoxLayout* vl0 = new QVBoxLayout(right_widget);
    vl0->setSpacing(0);
    vl0->setContentsMargins(0, 0, 0, 0);
    vl0->addWidget(m_nowplayingview);
    vl0->addWidget(m_playlistwidget);
    
    //! left pane
    QFrame* left_widget = new QFrame(this);
    left_widget->setFrameShape(QFrame::NoFrame);
    left_widget->setFrameShadow(QFrame::Plain);
    left_widget->setMinimumWidth(50);
    left_widget->setMaximumWidth(200);
    left_widget->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Expanding );
    
    QVBoxLayout* vl1 = new QVBoxLayout(left_widget);
    vl1->setSpacing(0);
    vl1->setContentsMargins(0, 0, 0, 0);
    vl1->addWidget(m_menuView);
    
    //! splitter population
    m_viewsSplitter->addWidget(left_widget);
    m_viewsSplitter->addWidget(m_graphicviewwidget);
    m_viewsSplitter->addWidget(right_widget);

    QMap<ENUM_ACTION, QAction*> *actions = ACTIONS();
    
    new SplitterCollapser(m_viewsSplitter, left_widget,  actions->value(APP_SHOW_MENU));
    new SplitterCollapser(m_viewsSplitter, right_widget, actions->value(APP_SHOW_PLAYQUEUE));
    
    mainFrameLayout->addWidget(m_viewsSplitter);

    centralWidgetLayout->addWidget(topFrame);
    centralWidgetLayout->addWidget(mainFrame);
}

  
//! --------- restoreState -----------------------------------------------------
void CentralWidget::restoreState()
{
    slot_show_playlist();
    slot_show_menu();
    slot_show_nowplaying();
    
    // splitter state
    if(!SETTINGS()->_splitterState.isEmpty()) {
      m_viewsSplitter->restoreState(SETTINGS()->_splitterState);
    }
    else {
      QList<int> list;
      list << 180 << 600 << 320;
      m_viewsSplitter->setSizes (list); // 1100
    }
    
    // Mode
    m_mainMode = MAIN::ViewMode (SETTINGS()->_viewMode);
    m_centraltoolbar->updateMode(m_mainMode);

    //! restore Model Index
    const QString radiofile = SETTINGS()->_radiofile;
    QModelIndex idx = static_cast<MenuModel*>(m_menuView->model())->restoreModelIdx(m_mainMode,radiofile);

    if(idx.isValid()){
      m_centraltoolbar->updateCurrentIndex(idx);
      m_menuView->updateSelection(idx);
      //Debug::debug() << "-- CentralWidget -> restore Model Index : " << idx;      
    }
}



//! --------- slot_modelitem_activated -----------------------------------------
void CentralWidget::slot_modelitem_activated(QModelIndex index)
{
    updateMenu(index, true);
}


void CentralWidget::slot_prevnext_activated(QModelIndex index, QString search)
{
    m_graphicviewwidget->setSearch(search);
    updateMenu(index, false);
}


void CentralWidget::updateMenu(QModelIndex index, bool newActivation)
{
    //Debug::debug() << "### CentralWidget -> updateMenu"; 
    
    if(!index.isValid()) return;

    m_menuView->updateSelection(index);

    //! breadcrumb updated only if activation not from prev/next
    if(newActivation)
      m_centraltoolbar->updateCurrentIndex(index);
    
    //! radio specific file 
    SETTINGS()->_radiofile = index.data(FileRole).toString();

    //! action triggering 
    QAction *g_action = index.data(GlobalActionRole).value<QAction*>();
      
    if(g_action) {
      m_mainMode = MAIN::ViewMode(g_action->data().toInt());
      m_centraltoolbar->updateMode(m_mainMode);
      g_action->trigger();
    } 
}


   
//! --------- Hide/Show Playlist Widget ----------------------------------------
void CentralWidget::slot_show_playlist( ) 
{
    SETTINGS()->_showPlayQueuePanel = ACTIONS()->value(APP_SHOW_PLAYQUEUE)->isChecked();
  
    if(SETTINGS()->_showPlayQueuePanel)
      m_viewsSplitter->widget(2)->show();
    else
      m_viewsSplitter->widget(2)->hide();
}

//! --------- Hide/Show Menu Widget --------------------------------------------  
void CentralWidget::slot_show_menu( ) 
{
    SETTINGS()->_showMenuPanel = ACTIONS()->value(APP_SHOW_MENU)->isChecked();
      
    if( SETTINGS()->_showMenuPanel )
      m_viewsSplitter->widget(0)->show();
    else
      m_viewsSplitter->widget(0)->hide();
}

//! --------- Hide/Show Now playing (m_nowplayingview) -------------------------  
void CentralWidget::slot_show_nowplaying( ) 
{
    SETTINGS()->_showNowPlaying= ACTIONS()->value(APP_SHOW_NOW_PLAYING)->isChecked();
      
    if( SETTINGS()->_showNowPlaying)
      m_nowplayingview->show();
    else
      m_nowplayingview->hide();
}

