/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "menuview.h"
#include "menumodel.h"
#include "Database/databasemanager.h"
#include "global_actions.h" 
//#include "debug.h" 

/*
********************************************************************************
*                                                                              *
*    Class MenuView                                                            *
*                                                                              *
********************************************************************************
*/
MenuView::MenuView(MenuModel *model, QWidget *parent) : QTreeView(parent)
{
    //! config palette
    QPalette palette;
    palette.setColor(QPalette::Background, Qt::white);
    palette.setColor(QPalette::Window , Qt::white);
    palette.setColor(QPalette::Base, Qt::white);
    //palette.setColor(QPalette::Text, QColor(Qt::black));
    this->setPalette(palette);
    
    //! config QTreeView
    this->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Expanding);
    this->setMinimumWidth(20);
    this->setAutoFillBackground(false);
    this->setFrameShape(QFrame::NoFrame);    
    this->setFrameShadow(QFrame::Plain);
    this->setContentsMargins(0,0,0,0);
  
    this->header()->hide();
    this->setWordWrap(true);
    this->setSelectionMode(QAbstractItemView::SingleSelection);
    this->setRootIsDecorated(false);
    this->setUniformRowHeights(false);
    
    //! vertical scrolbar setup
    this->verticalScrollBar()->setContextMenuPolicy(Qt::NoContextMenu);
    
    //! model & delegate
    this->setItemDelegate(new MenuViewDelegate(this));
    this->setModel(model); 
    
    //! default expand/collapse
    this->expandAll();
    
    //! selection model
    m_selection_model = new QItemSelectionModel(model);
    this->setSelectionModel(m_selection_model);
  
    //! signals
    connect(this,SIGNAL(clicked(QModelIndex)),model, SIGNAL(modelItemActivated(QModelIndex)));
}


/*******************************************************************************
    updateSelection
*******************************************************************************/
void MenuView::updateSelection(QModelIndex& index)
{
    //Debug::debug() << "-- MenuView --> updateSelection" << index;
    m_selection_model->select(index, QItemSelectionModel::ClearAndSelect);
}


/*******************************************************************************
    contextMenuEvent
*******************************************************************************/
void MenuView::contextMenuEvent ( QContextMenuEvent * event ) 
{
    QMenu *m = new QMenu(this);
    m->addAction(ACTIONS()->value(APP_SHOW_MENU));    

    m->popup(mapToGlobal(event->pos()));
    //m->deleteLater;
}

/*******************************************************************************
    drawBranches
*******************************************************************************/
void MenuView::drawBranches ( QPainter * painter, const QRect & rect, const QModelIndex & index ) const
{
    QRect primitive(rect.right() + 4, rect.top(), 5, rect.height());

    QStyleOption opt;
    opt.initFrom(this);
    opt.rect = primitive;
    
    const bool children = static_cast<QStandardItemModel*>( this->model() )->hasChildren(index);
    
    if(children) {
      const bool expanded = isExpanded (  index );
      if(!expanded)
        style()->drawPrimitive(QStyle::PE_IndicatorArrowRight, &opt, painter, this);
      else 
        style()->drawPrimitive(QStyle::PE_IndicatorArrowDown, &opt, painter, this);
    }
}

/*
********************************************************************************
*                                                                              *
*    Class MenuViewDelegate                                                    *
*                                                                              *
********************************************************************************
*/
MenuViewDelegate::MenuViewDelegate(QObject *parent):QItemDelegate(parent)
{
}

void MenuViewDelegate::paint ( QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index ) const
{
    if(!index.isValid() ) return;
    
    const bool children =  index.model()->hasChildren(index);
    
    //! get item data
    const QString text = index.data(Qt::DisplayRole).toString();
    const QIcon   icon = qvariant_cast<QIcon> ( index.data(Qt::DecorationRole) );

    //! font
    // a priori ce n'est pas nécessaire de faire appel a QFontDatabase
    //QFontDatabase * fontdb = new QFontDatabase();    
    //QFont boldFont   = fontdb->font( "Segoe UI", "Bold", 10); 
    //QFont normalFont = fontdb->font( "Segoe UI", "Normal", 9);
        
    QFont boldFont   = QFont("Segoe UI",10, QFont::Bold); 
    QFont normalFont = QFont("Segoe UI",9, QFont::Normal);
    
    boldFont.setStyleStrategy(QFont::PreferAntialias);
    normalFont.setStyleStrategy(QFont::PreferAntialias);
    
    painter->setRenderHint(QPainter::Antialiasing, true);

    if (!index.parent().isValid() && children)
    {
      //! paint text    
      painter->setFont(boldFont);
      painter->setPen( QColor(0x2e2b2b));
      painter->drawText(option.rect.adjusted(15 ,0,0,0),Qt::AlignVCenter, text);
    }
    else if(!index.parent().isValid() && !children)
    {
      //! draw backgroud
      QStyleOptionViewItemV4 opt(option);
      opt.state |= QStyle::State_Active;
      opt.state |= QStyle::State_HasFocus;      
      QStyle *style = opt.widget ? opt.widget->style() : QApplication::style();
      style->drawPrimitive(QStyle::PE_PanelItemViewItem, &opt, painter, opt.widget);  

      //! paint icon
      QRect decorationRect = QRect(option.rect.topLeft(),QSize(18,18));
      icon.paint(painter, decorationRect.adjusted(18,4,18,4), option.decorationAlignment);

      //! paint text
      painter->setFont(normalFont);
      if(option.state & QStyle::State_Selected)
         painter->setPen( QColor("white"));
      else
        painter->setPen( QColor(0x2e2b2b));
      
      painter->drawText(option.rect.adjusted(38,0,0,0),Qt::AlignVCenter, text);      
    }
    else //! parent valid --> it's a child
    {
      //! draw backgroud
      QStyleOptionViewItemV4 opt(option);
      opt.rect = opt.rect.adjusted(-20,0,0,0);
      opt.state |= QStyle::State_Active;
      opt.state |= QStyle::State_HasFocus;      
      QStyle *style = opt.widget ? opt.widget->style() : QApplication::style();
      style->drawPrimitive(QStyle::PE_PanelItemViewItem, &opt, painter, opt.widget);  

      //! paint icon
      QRect decorationRect = QRect(option.rect.topLeft(),QSize(18,18));
      icon.paint(painter, decorationRect.adjusted(-2,4,-2,4), option.decorationAlignment);

      //! paint text
      painter->setFont(normalFont);
      painter->setFont(normalFont);
      if(option.state & QStyle::State_Selected)
         painter->setPen( QColor("white"));
      else
        painter->setPen( QColor(0x2e2b2b));
      
      painter->drawText(option.rect.adjusted(20 ,0,0,0),Qt::AlignVCenter, text);
    }
}

/*******************************************************************************
    sizeHint
*******************************************************************************/
QSize MenuViewDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    if(!index.isValid()) 
      return QSize();
    else
      return QSize(option.rect.width(), 26);
}
