/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "histomodel.h"
#include "MediaItem/mediaitem.h"
#include "Database/database.h"
#include "EnginePlayer/engineplayer.h"
#include "collectionmodel.h"
#include "debug.h"


//! Qt
#include <QFile>
#include <QSqlQuery>
#include <QSqlDriver>

HistoModel* HistoModel::INSTANCE = 0;

/*
********************************************************************************
*                                                                              *
*    Class HistoModel                                                          *
*                                                                              *
********************************************************************************
*/
HistoModel::HistoModel(QObject *parent) : QObject(parent) 
{
    Debug::debug() << " ---- HistoModel --> creation";
    INSTANCE            = this;
    m_filter_pattern    = "";
    m_playing_item_idx  = -1;    
}

void HistoModel::clear()
{
    //! Do not delete item because we use Collection Model item
    MEDIA::qResetAll(m_listMediaItem);
    
    m_listMediaItem.clear();
    m_playing_item_idx  = -1;    
}


//! ------------------ HistoModel::updateModel ---------------------------------
void HistoModel::updateModel()
{
    Debug::debug() << " ---- HistoModel --> updateModel";
    this->clear();
    
    Database db;
    if (!db.connect("Histo")) {
        return;
    }
    
    QSqlQuery query("SELECT id,url,name,date, \
                            track_id,trackname,number,length,playcount,albumgain,albumpeakgain,trackgain,trackpeakgain, \
                            artist_name,album_name,genre_name,year FROM view_histo ORDER BY date DESC",*db.sqlDb());
    
    while (query.next())
    {
      MediaItemPtr media = MediaItemPtr(0);
      
      QVariant track_id = query.value(4);
      QString url       = query.value(1).toString();
      
      if(!track_id.isNull()) 
      {
          // increment pointer counter of mediaItem
          media = MediaItemPtr(
               CollectionModel::instance()->trackItemHash.value(track_id.toInt())
               ); 
          
          if(media) {
            media->data[TRACK::name]       = query.value(2); // keep name from histo
            media->data[TRACK::lastPlayed] = query.value(3);          
          }
      }
      else 
      {   
          //! track or stream not id database          
          if(MEDIA::isUrlLocal(url)) 
          {
            // do not search in file : to long for histo model !!
            media = MediaItemPtr(new MediaItem(TYPE_TRACK));
            media->data[TRACK::id]           = -1;
            media->data[TRACK::url]          = query.value(1);
            media->data[TRACK::name]         = query.value(2);
            media->data[TRACK::title]        = query.value(2);
            media->data[TRACK::lastPlayed]   = query.value(3);

            //! default value
            media->data[TRACK::isPlaying]    =  false;
            media->data[TRACK::isBroken]     =  QFile::exists(url) ? false : true;
            media->data[TRACK::isPlayed]     =  false;
            media->data[TRACK::isStopAfter]  =  false;
          }
          else 
          {
            media = MediaItemPtr(new MediaItem(TYPE_STREAM));
            media->data[STREAM::id]          = -1;
            media->data[STREAM::url]         = query.value(1);
            media->data[STREAM::name]        = query.value(2);
            media->data[STREAM::lastPlayed]  = query.value(3);
            media->data[STREAM::isFavorite]  = false;
            media->data[STREAM::isPlaying]   = false;
            media->data[STREAM::isBroken]    = false;
            media->data[STREAM::isPlayed]    = false;
            media->data[STREAM::isStopAfter] = false;	    
          }
      }
      
      if( media != MediaItemPtr(0) ) {
          m_mutex.lock();
          m_listMediaItem.append(media);
          m_mutex.unlock();
      }   
    } // end while
    
    QString currentTrack = EnginePlayer::instance()->enginePlayingUrl();
    if(!currentTrack.isEmpty()) {
        updatePlayingItem(currentTrack);
    }
}

//!------------HistoModel::getItem ---------------------------------------------
MediaItemPtr HistoModel::getItem(int row) 
{
   if (row < 0 || row >= m_listMediaItem.size())
        return MediaItemPtr(0);

   return m_listMediaItem.at(row);
}

//! --------- playing item method ----------------------------------------------
void HistoModel::updatePlayingItem(const QString& url)
{
    //Debug::debug() << " ---- HistoModel --> updatePlayingItem" << url;

    for ( int i = 0; i < m_listMediaItem.size(); i++ )
    {
      MediaItemPtr mi = m_listMediaItem.at(i);
      if (url ==  mi->data.at(TRACK::url).toString()) //! works STREAM or TRACK
      {
        if(mi->type == TYPE_TRACK)
          mi->data[TRACK::isPlaying]         = true;
        else
          mi->data[STREAM::isPlaying]        = true;
       
        m_playing_item_idx = i;
        emit signalPlayingStatusChanged();
        break;
      }
    }
}

void HistoModel::stopPlayingItem()
{
    if( m_playing_item_idx == -1) return;
  
    MediaItemPtr active_item = m_listMediaItem.at( m_playing_item_idx );
      
    if(active_item->type == TYPE_TRACK)
      active_item->data[TRACK::isPlaying]         = false;
    else
      active_item->data[STREAM::isPlaying]        = false;
       
    m_playing_item_idx = -1;
}  
  
//! --------- Filtering method -------------------------------------------------
bool HistoModel::isItemFiltered(const int row)
{
    if(m_filter_pattern.isEmpty()) return true;
  
    MediaItemPtr mi = getItem(row);
  
    if ( mi->type == TYPE_STREAM && mi->data.at(STREAM::name).toString().contains(m_filter_pattern, Qt::CaseInsensitive) )
      return true;
    else if ( mi->type == TYPE_TRACK && mi->data.at(TRACK::name).toString().contains(m_filter_pattern, Qt::CaseInsensitive) )
      return true;
  
    return false;
}


//! --------- broken item method -----------------------------------------------
void HistoModel::updateBrokenItem(const QString& url)
{
    //Debug::debug() << " ---- HistoModel --> updateBrokenItem" << url;

    for ( int i = 0; i < m_listMediaItem.size(); i++ )
    {
      MediaItemPtr mi = m_listMediaItem.at(i);    
    
      if (url ==  mi->data.at(TRACK::url).toString()) //! works STREAM or TRACK
      {
        if(mi->type == TYPE_TRACK)
          mi->data[TRACK::isBroken]         = true;
        else
          mi->data[STREAM::isBroken]        = true;
       
        emit signalPlayingStatusChanged();
        break;
      }
    }
}
