/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "ContextView/contextscene.h"
#include "EnginePlayer/engineplayer.h"

/*   info providers   */
#include "provider_lyrics.h"
#include "provider_lastfm.h"
#include "provider_echonest.h"
#include "provider_discogs.h"

#include "utilities.h"
#include "debug.h"

/*
********************************************************************************
*                                                                              *
*    Class ContextScene                                                        *
*                                                                              *
********************************************************************************
*/ 
ContextScene::ContextScene(QWidget* parent)
{
    m_parentWidget = parent;
    this->setParent(parent);
    this->setItemIndexMethod(QGraphicsScene::NoIndex);
    
    //! set engine player for update
    m_engine_player = EnginePlayer::instance();
    connect(m_engine_player, SIGNAL(playingInfoChanged()), this, SLOT(slot_update_scene()));
    connect(m_engine_player, SIGNAL(engineStateChanged()), this, SLOT(slot_update_scene()));

    //! init contexte scene
    m_activated     = false;
    m_initialized   = false;

    m_current_media = MediaItemPtr(0);
    m_request_id    = 0;
    is_new_artist   = false;
    is_new_album    = false;
    is_new_track    = false;    
}

ContextScene::~ContextScene()
{
    clear();
}
/*******************************************************************************
  ContextScene::startContextScene
    -> called from graphicsview widget (contextView action on menu triggered)
*******************************************************************************/
void ContextScene::startContextScene()
{
    setActivate(true);
    
    if( !m_initialized )
      init_scene();

    slot_update_scene();
}

/*******************************************************************************
  ContextScene::init_scene
*******************************************************************************/
void ContextScene::init_scene()
{
    Debug::debug() << "ContextScene::init_scene ";
  
    //!  init providers 
    m_map_providers.insert("discogs",  new DiscogsProvider(0) );
    m_map_providers.insert("echonest", new EchonestProvider(0) );
    m_map_providers.insert("lastfm",   new LastFmProvider(0) );
    m_map_providers.insert("lyrics",   new LyricsProvider(0) );
    
    Debug::debug() << "ContextScene::init providers";
    foreach (InfoProvider* provider, m_map_providers.values() ) {
      Debug::debug() << "ContextScene::init provider : " << provider->name();
      provider->init();
    }

    //!  Playing scene widgets
    m_artist_info_widget  = new ArtistInfoWidget(m_parentWidget, static_cast<EchonestProvider*>(m_map_providers.value("echonest")));
    m_similar_info_widget = new ArtistSimilarWidget(m_parentWidget, static_cast<EchonestProvider*>(m_map_providers.value("echonest")));
    m_disco_info_widget   = new DiscoInfoWidget(m_parentWidget, static_cast<DiscogsProvider*>(m_map_providers.value("discogs")));
    m_album_info_widget   = new AlbumInfoWidget(m_parentWidget, static_cast<LastFmProvider*>(m_map_providers.value("lastfm")));
    m_lyrics_info_widget  = new LyricsInfoWidget(m_parentWidget,static_cast<LyricsProvider*>( m_map_providers.value("lyrics") ) );
  
    connect(m_artist_info_widget, SIGNAL(updated()), this, SLOT(slot_update_draw()));
    connect(m_similar_info_widget, SIGNAL(updated()), this, SLOT(slot_update_draw()));
    connect(m_disco_info_widget, SIGNAL(updated()), this, SLOT(slot_update_draw()));
    connect(m_album_info_widget, SIGNAL(updated()), this, SLOT(slot_update_draw()));
    connect(m_lyrics_info_widget, SIGNAL(updated()), this, SLOT(slot_update_draw()));
    
    m_artist_info_widget->setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred );
    m_similar_info_widget->setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred );
    m_disco_info_widget->setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred );
    m_album_info_widget->setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred );
    m_lyrics_info_widget->setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred );

    m_layout    = new QGraphicsLinearLayout(Qt::Vertical);
    m_layout->setSpacing(15);
    m_layout->addItem(m_artist_info_widget);
    m_layout->addItem(m_similar_info_widget);
    m_layout->addItem(m_disco_info_widget);      
    m_layout->addItem(m_album_info_widget);
    m_layout->addItem(m_lyrics_info_widget);

    m_container = new QGraphicsWidget();
    m_container->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
    m_container->setLayout(m_layout);

    //! now playing widget
    m_nowplaying_info_widget = new NowPlayingInfoWidget(m_parentWidget);
    connect(m_engine_player, SIGNAL(streamInfoChanged()), m_nowplaying_info_widget, SLOT(update()));  

    QGraphicsLinearLayout* m_layout_2    = new QGraphicsLinearLayout(Qt::Vertical);
    m_layout_2->setSpacing(15);
    m_layout_2->addItem(m_nowplaying_info_widget);
    
    m_container_now_playing = new QGraphicsWidget();
    m_container_now_playing->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
    m_container_now_playing->setLayout(m_layout_2);
    
    //! add items to the scene    
    this->addItem(m_container_now_playing);
    this->addItem(m_container);
    m_container->hide();
    m_container_now_playing->show();    
    setSceneRect ( QRect(0,0,m_parentWidget->width(), m_parentWidget->height()));

    m_initialized = true;
}



/*******************************************************************************
  clear_scene
*******************************************************************************/
void ContextScene::clear_scene()
{
    Debug::debug() << "ContextScene::clear_scene ";
    m_container->hide();
    m_container_now_playing->hide();
}

/*******************************************************************************
  ContextScene::analyse_new_media
*******************************************************************************/
bool ContextScene::analyse_new_media()
{
    //Debug::debug() << "ContextScene::analyse_new_media " << m_engine_player->enginePlayingUrl();

    if(m_url == m_engine_player->enginePlayingUrl())
      return false;  // NO need to update the scene     
    else
      m_url = m_engine_player->enginePlayingUrl();
    
    is_new_artist = true;
    is_new_album  = true;
    is_new_track  = true; 
    
    // get new change
    MediaItemPtr player_media = m_engine_player->enginePlayingMediaItem();  
    
    if(m_current_media) {
      is_new_artist = (m_current_media->data.at(TRACK::artist).toString() == 
                       player_media->data.at(TRACK::artist).toString() ) ? false : true;

      is_new_album = (m_current_media->data.at(TRACK::album).toString() == 
                      player_media->data.at(TRACK::album).toString() ) ? false : true;

      is_new_track = (m_current_media->data.at(TRACK::title).toString() == 
                      player_media->data.at(TRACK::title).toString() ) ? false : true;
    
      m_current_media.reset();                      
      m_current_media = MediaItemPtr(0);
    }
        
    m_current_media =  MediaItemPtr(player_media);
    
    return true;
} 




/*******************************************************************************
  ContextScene::slot_update_scene
*******************************************************************************/
void ContextScene::slot_update_scene() 
{
    if(!isActivated()) return;
    Debug::debug() << "ContextScene::slot_update_scene ";
      
    /*-------------------------------------------------*/
    /*  PLAYING state                                  */
    /* ------------------------------------------------*/
    if(m_engine_player->engineState() != ENGINE::STOPPED && !m_engine_player->enginePlayingUrl().isEmpty())
    {
      if( !analyse_new_media() )
        return;
    
      clear_scene();

      if(m_current_media->type == TYPE_STREAM)
      {
        m_container_now_playing->show();
        return;
      }      

      if(!m_current_media) {
        Debug::error() << "ContextScene::ERROR updating mediaitem";
        return;
      }      
      
      //! initialisation des info
      m_request_id++;    
    
      foreach (InfoProvider* provider, m_map_providers.values()) {
        provider->cancel_searches();
        provider->set_media(m_current_media);
      }
      
      if(is_new_artist) {
        Debug::warning() << "ContextScene -> new artist";
        m_artist_info_widget->clear();
        m_similar_info_widget->clear();
        m_disco_info_widget->clear();
        
        m_artist_info_widget->set_artist_name( m_current_media->data.at(TRACK::artist).toString() );
        
        m_artist_info_widget->set_request_id(m_request_id);
        m_similar_info_widget->set_request_id(m_request_id);
        m_disco_info_widget->set_request_id(m_request_id);

        m_map_providers.value("echonest")->fetch_info(m_request_id);
        m_map_providers.value("discogs")->fetch_info(m_request_id);
      }

      if(is_new_album) {
        Debug::warning() << "ContextScene -> new album";
        m_album_info_widget->clear(); 

        m_album_info_widget->set_album_name( m_current_media->data.at(TRACK::album).toString() );
        m_album_info_widget->set_album_pix( m_engine_player->enginePlayingPixmap() );
      
        m_album_info_widget->set_request_id(m_request_id);
        m_map_providers.value("lastfm")->fetch_info(m_request_id);
      }
              
      if(is_new_track) {
        Debug::warning() << "ContextScene -> new track";
        m_lyrics_info_widget->clear();
        m_lyrics_info_widget->set_song_name( m_current_media->data.at(TRACK::title).toString() );	
        m_lyrics_info_widget->set_request_id(m_request_id);
        m_map_providers.value("lyrics")->fetch_info(m_request_id);
      }
      
      m_container->show();   
      slot_update_draw();      
    }
    /*-------------------------------------------------*/
    /*  STOPPED state                                  */
    /* ------------------------------------------------*/
    //! with VLC add (m_engine_player->enginePlayingUrl().isEmpty()) CONDITION
    else if(m_engine_player->engineState() == ENGINE::STOPPED && m_engine_player->enginePlayingUrl().isEmpty())
    {
      //Debug::warning() << "ContextScene -> ENGINE::STOPPED";
      clear_scene();
      m_url.clear();
      m_container_now_playing->show();
      
      if(m_current_media) {
        m_current_media.reset();
        m_current_media = MediaItemPtr(0);
      }
      
      setSceneRect ( QRect(0,0,m_parentWidget->width(), m_parentWidget->height()));
      return;
    }
}

  
/*******************************************************************************
  ContextScene::slot_update_draw
*******************************************************************************/
void ContextScene::slot_update_draw()
{
    Debug::debug() << "ContextScene::slot_update_draw ";
    m_layout->invalidate();
    m_container->update();

    int diff = m_container->size().height() - itemsBoundingRect().height() + 40;
    int ajust  = ( diff > 0) ? diff : 0; 
    setSceneRect ( itemsBoundingRect().adjusted(-10, -10, +10, +10 + ajust) );
}
