/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Dialog/mediaitem_edit_dialog.h"
#include "Database/database.h"
#include "MediaItem/mediaitem.h"
#include "utilities.h"
#include "debug.h"

#include "CollectionModel/collectionmodel.h"

#include <QSqlField>
#include <QSqlQuery>
#include <QSqlDriver>
#include <QVariant>
#include <QString>
#include <QtGui>

//! -------------- recupCoverArt -----------------------------------------------
//! copy old cover if exist into new cover art (according to new 
//! artist or album name)
void recupCoverArt(const QString &newCoverName, const QString &oldCoverName);


void recupCoverArt(const QString &newCoverName, const QString &oldCoverName)
{
    //Debug::debug() << " ----recupCoverArt newCoverName + oldCoverName---- " << newCoverName + " - " + oldCoverName;  
    QString storageLocation = UTIL::CONFIGDIR + "/albums/";
  
    //! quit if same filename
    if(newCoverName == oldCoverName) return;
   
    //! remove new cover if already exist
    QFile newCoverFile(storageLocation + newCoverName);
    if(newCoverFile.exists()) newCoverFile.remove();

    //! copy old into new
    QFile oldCoverFile(storageLocation + oldCoverName);
    if(oldCoverFile.exists()) { 
      //Debug::debug() << " ----recupCoverArt oldCoverName exists COPY ---- ";  
      oldCoverFile.rename(storageLocation + newCoverName);
    }
}


/*
********************************************************************************
*                                                                              *
*    Class MediaItem_Edit_Dialog                                               *
*                                                                              *
********************************************************************************
*/
MediaItem_Edit_Dialog::MediaItem_Edit_Dialog(T_DIALOG_TYPE type)
{
    Debug::debug() << "MediaItem_Edit_Dialog";
  
    dialog_type_  = type;
    isChanged_    = false;
    media_        = MediaItemPtr( 0 );
    
    //! create gui
    create_gui();
    
    //! Connection
    QObject::connect(ui_buttonBox, SIGNAL(accepted()), this, SLOT(on_buttonBox_accepted()));
    QObject::connect(ui_buttonBox, SIGNAL(rejected()), this, SLOT(on_buttonBox_rejected()));
    QObject::connect(ui_clear_playcount, SIGNAL(clicked()), this, SLOT(slot_clear_playcount()));
    
    if(dialog_type_ != DIALOG_TRACK)
    {
      QObject::connect(ui_clear_rating, SIGNAL(clicked()), this, SLOT(slot_clear_rating()));
    }
}

MediaItem_Edit_Dialog::~MediaItem_Edit_Dialog()
{
    media_.reset();
}

void MediaItem_Edit_Dialog::setMediaItem(MediaItemPtr media)
{
    media_     = MediaItemPtr( media );
    //Debug::debug() << "MediaItem_Edit_Dialog setMediaItem : " << media_;
    
    //! restore values from MediaItem
    if(dialog_type_ == DIALOG_ARTIST)
    {
      ui_artist_name->setText( media_->data.at(ARTIST::name).toString() );
      ui_playcount->setValue( media_->data.at(ARTIST::playcount).toInt() );
      
      set_rating(media_->data.at(ARTIST::isUserRating).toBool());
    } 
    else if(dialog_type_ == DIALOG_ALBUM) 
    {
      ui_album_name->setText( media_->data.at(ALBUM::name).toString() );
      ui_artist_name->setText( media_->parent()->data.at(ARTIST::name).toString() );
      ui_album_year->setDate( QDate(media_->data.at(ALBUM::year).toInt(), 1, 1) );
      ui_playcount->setValue( media_->data.at(ALBUM::playcount).toInt() );

      set_rating(media_->data.at(ALBUM::isUserRating).toBool());
    }
    else if(dialog_type_ == DIALOG_GENRE)
    {
      ui_album_name->setText( media_->data.at(ALBUM::name).toString() );
      ui_artist_name->setText( media_->parent()->data.at(ARTIST::name).toString() );
      ui_album_year->setDate( QDate(media_->data.at(ALBUM::year).toInt(), 1, 1) );
      ui_album_genre->setText( genre_ );
      ui_playcount->setValue( media_->data.at(ALBUM::playcount).toInt() );   

      set_rating(media_->data.at(ALBUM::isUserRating).toBool());
    }
    else if(dialog_type_ == DIALOG_TRACK)
    {
      ui_filename->setText( media_->data.at(TRACK::url).toString() );
      ui_track_title->setText( media_->data.at(TRACK::title).toString() );
      ui_album_name->setText( media_->data.at(TRACK::album).toString() );
      ui_artist_name->setText( media_->data.at(TRACK::artist).toString() );
      ui_track_year->setDate( QDate(media_->parent()->data.at(ALBUM::year).toInt(), 1, 1) );
      ui_track_genre->setText( media_->data.at(TRACK::genre).toString() );
      ui_track_number->setText( media_->data.at(TRACK::num).toString() );
      ui_playcount->setValue( media_->data.at(TRACK::playcount).toInt() );  

      set_rating(true);
    } 
    
    QObject::connect(ui_rating, SIGNAL(RatingChanged(float)), this, SLOT(slot_rating_changed(float)));
}

void MediaItem_Edit_Dialog::setGenre(const QString& genre)
{
    // shall be done before setMediaItem
    genre_     = genre;
}


/*******************************************************************************
   slot_clear_playcount
*******************************************************************************/
void MediaItem_Edit_Dialog::slot_clear_playcount()
{
    ui_playcount->setValue(0);  
}

/*******************************************************************************
   slot_rating_changed
*******************************************************************************/
void MediaItem_Edit_Dialog::slot_rating_changed(float rating)
{
Q_UNUSED(rating)
  //Debug::debug() << " MediaItem_Edit_Dialog::slot_rating_changed";
  //set_rating(true);
  switch (media_->type) {
      case TYPE_ARTIST : media_->data[ARTIST::isUserRating] = true; break;
      case TYPE_ALBUM  : media_->data[ALBUM::isUserRating]  = true; break;
      default: break;
    } 
    
    if( ui_clear_rating ) 
    {
        ui_clear_rating->setText(tr("clear user rating") );
        ui_clear_rating->setEnabled (true);
    }
}

/*******************************************************************************
   slot_clear_rating
*******************************************************************************/
void MediaItem_Edit_Dialog::slot_clear_rating()
{
    set_rating( false );
}

void MediaItem_Edit_Dialog::set_rating(bool isUser)
{
    //Debug::debug() << "set_rating USER : " << isUser;
    switch (media_->type) {
      case TYPE_ARTIST : media_->data[ARTIST::isUserRating] = isUser; break;
      case TYPE_ALBUM  : media_->data[ALBUM::isUserRating]  = isUser; break;
      default: break;
    }      

    if(isUser) 
    {
      switch (media_->type) {
        case TYPE_ARTIST : ui_rating->set_rating( media_->data.at(ARTIST::rating).toFloat() );
        case TYPE_ALBUM  : ui_rating->set_rating( media_->data.at(ALBUM::rating).toFloat() ); break;
        case TYPE_TRACK  : ui_rating->set_rating( media_->data.at(TRACK::rating).toFloat() ); break;
        default: break;
      }      
      
      if( ui_clear_rating ) 
      {
        ui_clear_rating->setText(tr("clear user rating") );
        ui_clear_rating->setEnabled (true);
      }
    }
    else 
    {
      float auto_rating = CollectionModel::instance()->getItemAutoRating(media_);
      ui_rating->set_rating( auto_rating  );
      
      if( ui_clear_rating ) 
      {
        ui_clear_rating->setText(tr("auto rating"));
        ui_clear_rating->setEnabled (false);
      }
    }
}


/*******************************************************************************
    GUI creation
*******************************************************************************/
void MediaItem_Edit_Dialog::create_gui()
{
    switch(dialog_type_)
    {
      case DIALOG_ARTIST : create_gui_artist(); break;
      case DIALOG_ALBUM : create_gui_album(); break;
      case DIALOG_GENRE : create_gui_genre(); break;
      case DIALOG_TRACK : create_gui_track(); break;
      default:break;
    }
}

void MediaItem_Edit_Dialog::create_gui_artist()
{
    this->resize(445, 150);
    this->setWindowTitle(QString::fromUtf8("YaRock - Edit artist"));
    
    ui_buttonBox = new QDialogButtonBox();
    ui_buttonBox->setGeometry(QRect(140, 110, 291, 32));
    ui_buttonBox->setOrientation(Qt::Horizontal);
    ui_buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

    ui_artist_name = new QLineEdit();
    ui_playcount   = new QSpinBox();
    ui_rating      = new RatingWidget();
    
    ui_clear_playcount = new QPushButton(tr("clear playcount"));
    ui_clear_rating    = new QPushButton();
    
    ui_artist_name->setMinimumHeight(25);
    ui_playcount->setMinimumHeight(25);
    ui_rating->setMinimumHeight(25);
    
    ui_artist_name->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);
    ui_playcount->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);

    QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred);
    sizePolicy.setHeightForWidth(ui_rating->sizePolicy().hasHeightForWidth());
    ui_rating->setSizePolicy(sizePolicy);
    
    QGridLayout *gridLayout = new QGridLayout();
    gridLayout->setContentsMargins(0, 0, 0, 0);
    
    gridLayout->addWidget(new QLabel(tr("Artist")), 0, 0, 1, 1);
    gridLayout->addWidget(ui_artist_name, 0, 1, 1, 2);
    gridLayout->addWidget(new QLabel(tr("Playcount")), 1, 0, 1, 1);
    gridLayout->addWidget(ui_playcount, 1, 1, 1, 1);
    gridLayout->addWidget(ui_clear_playcount, 1, 2, 1, 1);    
    gridLayout->addWidget(new QLabel(tr("Rating")), 2, 0, 1, 1);
    gridLayout->addWidget(ui_rating, 2, 1, 1, 1);
    gridLayout->addWidget(ui_clear_rating, 2, 2, 1, 1);
   
    QVBoxLayout * layout = new QVBoxLayout(this);
    layout->addLayout(gridLayout);
    layout->addItem(new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding));       
    layout->addWidget(ui_buttonBox);
}

void MediaItem_Edit_Dialog::create_gui_album()
{
    this->resize(445, 150);
    this->setWindowTitle(QString::fromUtf8("YaRock - Edit album"));
    
    ui_buttonBox = new QDialogButtonBox();
    ui_buttonBox->setGeometry(QRect(140, 110, 291, 32));
    ui_buttonBox->setOrientation(Qt::Horizontal);
    ui_buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

    ui_album_name  = new QLineEdit();
    ui_artist_name = new QLineEdit();
    ui_album_year  = new QDateEdit();
    ui_playcount   = new QSpinBox();
    ui_rating      = new RatingWidget();
    
    ui_album_year->setDisplayFormat("yyyy");
    
    ui_clear_playcount = new QPushButton(tr("clear playcount"));
    ui_clear_rating    = new QPushButton();
    
    ui_album_name->setMinimumHeight(25);
    ui_artist_name->setMinimumHeight(25);
    ui_album_year->setMinimumHeight(25);
    ui_playcount->setMinimumHeight(25);
    ui_rating->setMinimumHeight(25);
    
    ui_album_name->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);
    ui_artist_name->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);
    ui_album_year->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);
    ui_playcount->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);

    QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred);
    sizePolicy.setHeightForWidth(ui_rating->sizePolicy().hasHeightForWidth());
    ui_rating->setSizePolicy(sizePolicy);
    
    
    QGridLayout *gridLayout = new QGridLayout();
    gridLayout->setContentsMargins(0, 0, 0, 0);
    
    gridLayout->addWidget(new QLabel(tr("Album")), 0, 0, 1, 1);
    gridLayout->addWidget(ui_album_name, 0, 1, 1, 2);
    gridLayout->addWidget(new QLabel(tr("Artist")), 1, 0, 1, 1);
    gridLayout->addWidget(ui_artist_name, 1, 1, 1, 2);
    gridLayout->addWidget(new QLabel(tr("Year")), 2, 0, 1, 1);
    gridLayout->addWidget(ui_album_year, 2, 1, 1, 2);
    gridLayout->addWidget(new QLabel(tr("Playcount")), 3, 0, 1, 1);
    gridLayout->addWidget(ui_playcount, 3, 1, 1, 1);
    gridLayout->addWidget(ui_clear_playcount, 3, 2, 1, 1);    
    gridLayout->addWidget(new QLabel(tr("Rating")), 4, 0, 1, 1);
    gridLayout->addWidget(ui_rating, 4, 1, 1, 1);
    gridLayout->addWidget(ui_clear_rating, 4,2,1,1);
    
    
    QVBoxLayout * layout = new QVBoxLayout(this);
    layout->addLayout(gridLayout);
    layout->addItem(new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding));     
    layout->addWidget(ui_buttonBox);
}

void MediaItem_Edit_Dialog::create_gui_genre()
{
    this->resize(445, 150);
    this->setWindowTitle(QString::fromUtf8("YaRock - Edit album"));
    
    ui_buttonBox = new QDialogButtonBox();
    ui_buttonBox->setGeometry(QRect(140, 110, 291, 32));
    ui_buttonBox->setOrientation(Qt::Horizontal);
    ui_buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

    ui_album_name  = new QLineEdit();
    ui_artist_name = new QLineEdit();
    ui_album_year  = new QDateEdit();
    ui_album_genre = new QLineEdit();
    ui_playcount   = new QSpinBox();
    ui_rating      = new RatingWidget();
    
    ui_album_year->setDisplayFormat("yyyy");
    
    ui_clear_playcount = new QPushButton(tr("clear playcount"));
    ui_clear_rating    = new QPushButton();
    
    ui_album_name->setMinimumHeight(25);
    ui_artist_name->setMinimumHeight(25);
    ui_album_year->setMinimumHeight(25);
    ui_playcount->setMinimumHeight(25);
    ui_album_genre->setMinimumHeight(25);
    ui_rating->setMinimumHeight(25);
    
    ui_album_name->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);
    ui_artist_name->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);
    ui_album_year->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);
    ui_playcount->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);
    ui_album_genre->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);

    QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred);
    sizePolicy.setHeightForWidth(ui_rating->sizePolicy().hasHeightForWidth());
    ui_rating->setSizePolicy(sizePolicy);
    
    QGridLayout *gridLayout = new QGridLayout();
    gridLayout->setContentsMargins(0, 0, 0, 0);
    
    gridLayout->addWidget(new QLabel(tr("Album")), 0, 0, 1, 1);
    gridLayout->addWidget(ui_album_name, 0, 1, 1, 2);
    gridLayout->addWidget(new QLabel(tr("Artist")), 1, 0, 1, 1);
    gridLayout->addWidget(ui_artist_name, 1, 1, 1, 2);
    gridLayout->addWidget(new QLabel(tr("Year")), 2, 0, 1, 1);
    gridLayout->addWidget(ui_album_year, 2, 1, 1, 2);
    gridLayout->addWidget(new QLabel(tr("Genre")), 3, 0, 1, 1);
    gridLayout->addWidget(ui_album_genre, 3, 1, 1, 2);    
    gridLayout->addWidget(new QLabel(tr("Playcount")), 4, 0, 1, 1);
    gridLayout->addWidget(ui_playcount, 4, 1, 1, 1);
    gridLayout->addWidget(ui_clear_playcount, 4, 2, 1, 1);    
    gridLayout->addWidget(new QLabel(tr("Rating")), 5, 0, 1, 1);
    gridLayout->addWidget(ui_rating, 5, 1, 1, 1);
    gridLayout->addWidget(ui_clear_rating, 5,2,1,1);
    
    
    QVBoxLayout * layout = new QVBoxLayout(this);
    layout->addLayout(gridLayout);
    layout->addWidget(ui_buttonBox);
    layout->addItem(new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding));   
}

void MediaItem_Edit_Dialog::create_gui_track()
{
    this->resize(445, 150);
    this->setWindowTitle(QString::fromUtf8("YaRock - Edit track"));
    
    ui_buttonBox = new QDialogButtonBox();
    ui_buttonBox->setGeometry(QRect(140, 110, 291, 32));
    ui_buttonBox->setOrientation(Qt::Horizontal);
    ui_buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

    ui_filename     = new QLineEdit();
    ui_track_title  = new QLineEdit();
    ui_album_name   = new QLineEdit();
    ui_artist_name  = new QLineEdit();
    ui_track_year   = new QDateEdit();
    ui_track_genre  = new QLineEdit();
    ui_track_number = new QLineEdit();
    ui_playcount    = new QSpinBox();
    ui_rating       = new RatingWidget();
    
    ui_filename->setStyleSheet(QString::fromUtf8("QLineEdit {background: transparent;}"));
    ui_filename->setFrame(false);
    ui_filename->setReadOnly(true);
    
    ui_track_year->setDisplayFormat("yyyy");
    
    ui_clear_playcount = new QPushButton(tr("clear playcount"));
    ui_clear_rating    = 0;

    ui_track_title->setMinimumHeight(25);
    ui_album_name->setMinimumHeight(25);
    ui_artist_name->setMinimumHeight(25);
    ui_track_year->setMinimumHeight(25);
    ui_track_genre->setMinimumHeight(25);
    ui_track_number->setMinimumHeight(25);
    ui_playcount->setMinimumHeight(25);
    ui_rating->setMinimumHeight(25);
    
    ui_track_title->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);
    ui_album_name->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);
    ui_artist_name->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);
    ui_track_year->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);
    ui_track_genre->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);
    ui_track_number->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);
    ui_playcount->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);

    QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred);
    sizePolicy.setHeightForWidth(ui_rating->sizePolicy().hasHeightForWidth());
    ui_rating->setSizePolicy(sizePolicy);


    QGridLayout *gridLayout = new QGridLayout();
    gridLayout->setContentsMargins(0, 0, 0, 0);
    
    gridLayout->addWidget(new QLabel(tr("File")), 0, 0, 1, 1);
    gridLayout->addWidget(ui_filename, 0, 1, 1, 2);
    gridLayout->addWidget(new QLabel(tr("Title")), 1, 0, 1, 1);
    gridLayout->addWidget(ui_track_title, 1, 1, 1, 2);
    gridLayout->addWidget(new QLabel(tr("Album")), 2, 0, 1, 1);
    gridLayout->addWidget(ui_album_name, 2, 1, 1, 2);
    gridLayout->addWidget(new QLabel(tr("Artist")), 3, 0, 1, 1);
    gridLayout->addWidget(ui_artist_name, 3, 1, 1, 2);
    gridLayout->addWidget(new QLabel(tr("Year")), 4, 0, 1, 1);
    gridLayout->addWidget(ui_track_year, 4, 1, 1, 2);
    gridLayout->addWidget(new QLabel(tr("Genre")), 5, 0, 1, 1);
    gridLayout->addWidget(ui_track_genre, 5, 1, 1, 2);
    gridLayout->addWidget(new QLabel(tr("Number")), 6, 0, 1, 1);
    gridLayout->addWidget(ui_track_number, 6, 1, 1, 2);

    gridLayout->addWidget(new QLabel(tr("Playcount")), 7, 0, 1, 1);
    gridLayout->addWidget(ui_playcount, 7, 1, 1, 1);
    gridLayout->addWidget(ui_clear_playcount, 7, 2, 1, 1);

    gridLayout->addWidget(new QLabel(tr("Rating")), 8, 0, 1, 1);
    gridLayout->addWidget(ui_rating, 8, 1, 1, 1);

    
    QVBoxLayout * layout = new QVBoxLayout(this);
    layout->addLayout(gridLayout);
    layout->addWidget(ui_buttonBox);
    layout->addItem(new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding)); 
}
    

/*******************************************************************************
    on_buttonBox_rejected
*******************************************************************************/
void MediaItem_Edit_Dialog::on_buttonBox_rejected()
{
    this->setResult(QDialog::Rejected);
    QDialog::reject();
    this->close();
}
/*******************************************************************************
    on_buttonBox_accepted
*******************************************************************************/
void MediaItem_Edit_Dialog::on_buttonBox_accepted()
{
    Debug::debug() << "MediaItem_Edit_Dialog::on_buttonBox_accepted";

    switch(dialog_type_)
    {
      case DIALOG_ARTIST : do_changes_artist(); break;
      case DIALOG_ALBUM : do_changes_album(); break;
      case DIALOG_GENRE : do_changes_album_genre(); break;
      case DIALOG_TRACK : do_changes_track(); break;
      default:break;
    }
    
    this->setResult(QDialog::Accepted);
    QDialog::accept();
    this->close();
}

/*******************************************************************************
    do_changes_artist 
*******************************************************************************/
void MediaItem_Edit_Dialog::do_changes_artist()
{
    //! evaluate change
    if( media_->data.at(ARTIST::name).toString() == ui_artist_name->text() && 
        media_->data.at(ARTIST::playcount).toInt() == ui_playcount->value() &&
        ((media_->data.at(ARTIST::rating).toFloat() == ui_rating->rating())&& media_->data.at(ARTIST::isUserRating).toBool())
    )
    {
      Debug::debug() << "no change done --> close";
      isChanged_ = false;
      this->close();
      return;      
    }
    
    Debug::debug() << "change done --> apply";
    isChanged_ = true;

    
    //! apply new tag values
    QString  old_artist_name              = media_->data[ARTIST::name].toString();
    media_->data[ARTIST::name]            = ui_artist_name->text();
    media_->data[ARTIST::playcount]       = ui_playcount->value();
    media_->data[ARTIST::rating]          = media_->data.at(ARTIST::isUserRating).toBool() ? ui_rating->rating() : -1.0;
    
    //! apply changes to database
    QString command;
    Database db;
    if (!db.connect("MainThread")) return;
  
    QSqlQuery("BEGIN TRANSACTION;",*db.sqlDb());
    
    //! ARTIST part in database
    int new_artist_id;
    {
      command = QString("SELECT id FROM artists WHERE name='%1';").arg(media_->data.at(ARTIST::name).toString());    
      QSqlQuery query(command,*db.sqlDb());
      Debug::debug() << "database -> select artist : " << query.next();      
    
      if (!query.isValid())
      {
          QSqlQuery query(*db.sqlDb());
          query.prepare("INSERT INTO artists(name,favorite,playcount,rating) VALUES(?,?,?,?);");
          query.addBindValue(media_->data.at(ARTIST::name).toString());
          query.addBindValue(media_->data.at(ARTIST::isFavorite).toInt());
          query.addBindValue(media_->data.at(ARTIST::playcount).toInt());
          query.addBindValue(media_->data.at(ARTIST::rating).toFloat());
          Debug::debug() << "database -> insert artist : " << query.exec();     

          command = QString("SELECT id FROM artists WHERE name='%1';").arg(media_->data.at(ARTIST::name).toString());    
          QSqlQuery query2(command,*db.sqlDb());
          query2.next();
          new_artist_id = query2.value(0).toInt();	  
      }
      else
      {
          new_artist_id = query.value(0).toInt();

          command = QString("UPDATE artists SET name='%1',playcount=%2,rating=? WHERE id=%3;").arg(
            media_->data.at(ARTIST::name).toString(),
            QString::number(media_->data.at(ARTIST::playcount).toInt()),
            QString::number(new_artist_id) );

          QSqlQuery query(command, *db.sqlDb());
          query.addBindValue(media_->data.at(ARTIST::rating).toFloat());
  
          Debug::debug() << "database -> update artist : " << query.exec();
      }
      Debug::debug() << "database -> selected artist : " << new_artist_id;            
    }
    
    //! ALBUM Part in database
    {
      command = QString("SELECT id,name,cover FROM view_albums WHERE artist_name='%1';").arg(old_artist_name);
      QSqlQuery query(command, *db.sqlDb());      
      
      while(query.next()) 
      {
       QString old_cover_name = query.value(2).toString();
       //Debug::debug() << " ----database -> old_cover_name ---- " <<  old_cover_name;  

       QString new_cover_name = MEDIA::getCoverName(media_->data.at(ARTIST::name).toString(), query.value(1).toString());
       //Debug::debug() << " ----database -> new_cover_name ---- " <<  new_cover_name;  

       command = QString("UPDATE albums SET cover='%1',artist_id='%2' WHERE id='%3';").arg(
            new_cover_name,
            QString::number(new_artist_id),
            query.value(0).toString() );

       QSqlQuery query(command, *db.sqlDb());
       Debug::debug() << "database -> update album : " << query.exec();
    
       //! If old cover exist -> take it !!
       recupCoverArt(new_cover_name, old_cover_name);
       
     } //! WHILE album query
    }
    
    //! TRACK part in database
    {
      command = QString("UPDATE tracks SET artist_id='%1' WHERE artist_id='%2' ;").arg(
            QString::number(new_artist_id),
            QString::number(media_->data.at(ARTIST::id).toInt()) );
       
      QSqlQuery query(command, *db.sqlDb());
      Debug::debug() << "database -> update track : " << query.exec();       
    }
    
    //! Database Clean 
    QSqlQuery query3("DELETE FROM artists WHERE id NOT IN (SELECT artist_id FROM tracks GROUP BY artist_id);", *db.sqlDb());

    QSqlQuery("COMMIT TRANSACTION;",*db.sqlDb());

}

/*******************************************************************************
    do_changes_album 
*******************************************************************************/
void MediaItem_Edit_Dialog::do_changes_album()
{
    QSqlField data("col",QVariant::String);

    //! evaluate change
    if( media_->data.at(ALBUM::name).toString() == ui_album_name->text() && 
        media_->parent()->data.at(ARTIST::name).toString() == ui_artist_name->text()  &&
        QDate(media_->data.at(ALBUM::year).toInt(),1,1) == ui_album_year->date() &&
        media_->data.at(ALBUM::playcount).toInt() == ui_playcount->value() &&
        ((media_->data.at(ALBUM::rating).toFloat() == ui_rating->rating()) && media_->data.at(ALBUM::isUserRating).toBool()) 
    )
    {
      Debug::debug() << "no change done --> close";
      isChanged_ = false;
      this->close();
      return;      
    }
    
    Debug::debug() << "change done --> apply";
    isChanged_ = true;

  
    //! keep old values to be restored
    const QString  old_cover_name       = media_->data.at(ALBUM::coverpath).toString();
    
    //! apply new tag values
    media_->data[ALBUM::name]            = ui_album_name->text();
    media_->data[ALBUM::year]            = ui_album_year->date().year();
    media_->parent()->data[ARTIST::name] = ui_artist_name->text();
    media_->data[ALBUM::playcount]       = ui_playcount->value();
    media_->data[ALBUM::rating]          = media_->data.at(ALBUM::isUserRating).toBool() ? ui_rating->rating() : -1.0;

    foreach ( MediaItemPtr track, media_->childItems) 
    {
      track->data[TRACK::album]   = ui_album_name->text();
      track->data[TRACK::artist]  = ui_artist_name->text();
      track->data[TRACK::year]    = ui_album_year->date().year();
    }
    
    const QString  new_cover_name       = MEDIA::getCoverName(media_->parent()->data.at(ARTIST::name).toString(),media_->data.at(ALBUM::name).toString() );
    
 
    //! apply to database
    QString command;
    Database db;
    if (!db.connect("MainThread")) return;
    QSqlQuery("BEGIN TRANSACTION;",*db.sqlDb());

    //! YEAR part in database
    {
        command = QString("SELECT id FROM years WHERE year='%1';").arg(
          QString::number(media_->data.at(ALBUM::year).toUInt()) );      
          
        QSqlQuery query(command, *db.sqlDb());
        Debug::debug() << "database -> select year " << query.next();

        if (!query.isValid())
        {
          QSqlQuery query("INSERT INTO years(year) VALUES("+QString::number(media_->data.at(ALBUM::year).toUInt())+");", *db.sqlDb());
        
          Debug::debug() << "database -> insert year : " << query.exec();      
        }
    }
    
    int artist_id;
    //! ARTIST part in database
    {
      command = QString("SELECT id FROM artists WHERE name='%1';").arg(
        media_->parent()->data.at(ARTIST::name).toString() );
      
      QSqlQuery query(command, *db.sqlDb());
      Debug::debug() << "database -> select artist " << media_->parent()->data.at(ARTIST::name).toString() << ": " << query.next();
      
      if (!query.isValid()) 
      {
        QSqlQuery query(*db.sqlDb());
        query.prepare("INSERT INTO artists(name,favorite,playcount,rating) VALUES(?,?,?,?);");
        query.addBindValue(media_->parent()->data.at(ARTIST::name).toString());
        query.addBindValue(media_->parent()->data.at(ARTIST::isFavorite).toInt());
    
        int old_artist_playcount = media_->parent()->data.at(ARTIST::playcount).toInt();

        query.addBindValue(QString::number(old_artist_playcount));
        query.addBindValue(media_->parent()->data.at(ARTIST::rating).toFloat());
    
        Debug::debug() << "database -> insert artist : " << query.exec();

        QSqlQuery query2(command, *db.sqlDb());
        query2.next();
        artist_id = query2.value(0).toInt();	
      }
      else
      {
        artist_id = query.value(0).toInt();
      }

      Debug::debug() << "database -> selected artist_id " <<artist_id;
    }

    int album_id;
    //! ALBUM part in database
    {
      //! BUG FIX problem if string of album contain quote 
      //! WARNING le probleme est potentiellement present ailleurs
      //Debug::debug() << " media_->data.at(ALBUM::name) = " << media_->data.at(ALBUM::name).toString();

      data.setValue(media_->data.at(ALBUM::name).toString());
      QString album_name_sql = db.sqlDb()->driver()->formatValue(data,false);

      QSqlQuery query("SELECT `id` FROM `albums` WHERE `name`="+album_name_sql+" AND `artist_id`="+QString::number(artist_id)+";", *db.sqlDb());
      Debug::debug() << "database -> #1 select album/artist : " << query.next();      

      
      if (!query.isValid()) 
      {
          command = QString("INSERT INTO albums(name,artist_id,cover,year,favorite,playcount,rating) VALUES(?,'%1',?,?,?,?,?);").arg(
            QString::number(artist_id) );

          QSqlQuery query(*db.sqlDb());
          query.prepare(command);
          query.addBindValue( media_->data.at(ALBUM::name).toString() );
          query.addBindValue( new_cover_name );
          query.addBindValue( QString::number(media_->data.at(ALBUM::year).toUInt() ) );
          query.addBindValue( media_->data.at(ALBUM::isFavorite).toInt() );
          query.addBindValue( QString::number(media_->data.at(ALBUM::playcount).toInt()) );
          query.addBindValue( media_->data.at(ALBUM::rating).toFloat() );
      
          Debug::debug() << "database -> insert album : " << query.exec();
          
          QSqlQuery query2("SELECT `id` FROM `albums` WHERE `name`="+album_name_sql+" AND " \
                           " `artist_id`="+QString::number(artist_id)+";", *db.sqlDb());
      
          query2.next(); 
          album_id = query2.value(0).toInt();
          Debug::debug() << "database -> insert album album_id: " << album_id;
      }
      else 
      {
          album_id = query.value(0).toInt();
          command = QString("UPDATE albums  "  \
                      " SET name=?,         "  \
                      "     artist_id='%1', "  \
                      "     cover=?,        "  \
                      "     year=?,         "  \
                      "     playcount=?,    "  \
                      "     rating=? WHERE id='%2';").arg(
            QString::number(artist_id),
            QString::number(album_id) );

          QSqlQuery query(*db.sqlDb());
          query.prepare(command);
          query.addBindValue( media_->data.at(ALBUM::name).toString() );
          query.addBindValue( new_cover_name );
          query.addBindValue( QString::number(media_->data.at(ALBUM::year).toUInt()) );
          query.addBindValue( QString::number(media_->data.at(ALBUM::playcount).toInt()) );
          query.addBindValue( media_->data.at(ALBUM::rating).toFloat() );

          Debug::debug() << "database -> update album : " << query.exec();
      }

      Debug::debug() << "database -> selected album_id " << album_id;      
    } 
    
    //! TRACK part in database
    foreach ( MediaItemPtr track, media_->childItems)
    {    
        int trackId = track->data.at(TRACK::id).toInt();
            
        command = QString("UPDATE tracks "      \
                          "SET artist_id=%1, "  \
                          "    album_id=%2,  "   \
                          "    year_id=(SELECT id FROM years WHERE year='%3') " \
                          " WHERE id=%4;").arg(
           QString::number(artist_id),
           QString::number(album_id),
           QString::number(media_->data.at(ALBUM::year).toUInt()),
           QString::number(trackId) );

        QSqlQuery query(command, *db.sqlDb());
        Debug::debug() << "database -> update tracks id  " << trackId << " : " << query.exec();
    }

    //! Just once , If old cover exist -> take it !!
    recupCoverArt(new_cover_name,old_cover_name);
  
    //! Database Clean
    QSqlQuery queryClean1("DELETE FROM `albums` WHERE `id` NOT IN (SELECT `album_id` FROM `tracks` GROUP BY `album_id`);", *db.sqlDb());
    QSqlQuery queryClean2("DELETE FROM `genres` WHERE `id` NOT IN (SELECT `genre_id` FROM `tracks` GROUP BY `genre_id`);", *db.sqlDb());
    QSqlQuery queryClean3("DELETE FROM `artists` WHERE `id` NOT IN (SELECT `artist_id` FROM `tracks` GROUP BY `artist_id`);", *db.sqlDb());
    QSqlQuery queryClean4("DELETE FROM `years` WHERE `id` NOT IN (SELECT `year_id` FROM `tracks` GROUP BY `year_id`);", *db.sqlDb());

    QSqlQuery("COMMIT TRANSACTION;",*db.sqlDb());
}


/*******************************************************************************
    do_changes_album_genre 
*******************************************************************************/
void MediaItem_Edit_Dialog::do_changes_album_genre()
{
    QSqlField data("col",QVariant::String);

    //! evaluate change
    if( media_->data.at(ALBUM::name).toString() == ui_album_name->text() && 
        media_->parent()->data.at(ARTIST::name).toString() == ui_artist_name->text()  &&
        QDate(media_->data.at(ALBUM::year).toInt(),1,1) == ui_album_year->date() &&
        media_->data.at(ALBUM::playcount).toInt() == ui_playcount->value() &&
        genre_ == ui_album_genre->text() &&
        ((media_->data.at(ALBUM::rating).toFloat() == ui_rating->rating()) && media_->data.at(ALBUM::isUserRating).toBool()) 
    )
    {
      Debug::debug() << "no change done --> close";
      isChanged_ = false;
      this->close();
      return;      
    }
    
    Debug::debug() << "change done --> apply";
    isChanged_ = true;

  
    //! keep old values to be restored
    const QString  old_cover_name       = media_->data.at(ALBUM::coverpath).toString();
    
    //! apply new tag values
    media_->data[ALBUM::name]            = ui_album_name->text();
    media_->data[ALBUM::year]            = ui_album_year->date().year();
    media_->parent()->data[ARTIST::name] = ui_artist_name->text();
    media_->data[ALBUM::playcount]       = ui_playcount->value();
    media_->data[ALBUM::rating]          = media_->data.at(ALBUM::isUserRating).toBool() ? ui_rating->rating() : -1.0;
    
    foreach ( MediaItemPtr track, media_->childItems) 
    {
      track->data[TRACK::album]   = ui_album_name->text();
      track->data[TRACK::artist]  = ui_artist_name->text();
      track->data[TRACK::year]    = ui_album_year->date().year();
      track->data[TRACK::genre]   = ui_album_genre->text();
    }
    
    const QString  new_cover_name       = MEDIA::getCoverName(media_->parent()->data.at(ARTIST::name).toString(),media_->data.at(ALBUM::name).toString() );
    
 
    //! apply to database
    QString command;
    Database db;
    if (!db.connect("MainThread")) return;
    QSqlQuery("BEGIN TRANSACTION;",*db.sqlDb());

    
    //! GENRE part in database
    {
        command = QString("SELECT id FROM genres WHERE genre='%1';").arg(
          ui_album_genre->text() );

        QSqlQuery query(command, *db.sqlDb());
        Debug::debug() << "database -> select genre : " << query.next();      

        if (!query.isValid())
        {
          command = QString("INSERT INTO genres (genre) VALUES('%1');").arg(
            ui_album_genre->text() );
  
          QSqlQuery query(command, *db.sqlDb());
          Debug::debug() << "database -> insert genre : " << query.exec();      
        }
    }    
    
    //! YEAR part in database
    {
        command = QString("SELECT id FROM years WHERE year='%1';").arg(
          QString::number(media_->data.at(ALBUM::year).toUInt()) );      
          
        QSqlQuery query(command, *db.sqlDb());
        Debug::debug() << "database -> select year " << query.next();

        if (!query.isValid())
        {
          QSqlQuery query("INSERT INTO years(year) VALUES("+QString::number(media_->data.at(ALBUM::year).toUInt())+");", *db.sqlDb());
        
          Debug::debug() << "database -> insert year : " << query.exec();      
        }
    }
    
    int artist_id;
    //! ARTIST part in database
    {
      command = QString("SELECT id FROM artists WHERE name='%1';").arg(
        media_->parent()->data.at(ARTIST::name).toString() );
      
      QSqlQuery query(command, *db.sqlDb());
      Debug::debug() << "database -> select artist " << media_->parent()->data.at(ARTIST::name).toString() << ": " << query.next();
      
      if (!query.isValid()) 
      {
        QSqlQuery query(*db.sqlDb());
        query.prepare("INSERT INTO artists(name,favorite,playcount,rating) VALUES(?,?,?,?);");
        query.addBindValue(media_->parent()->data.at(ARTIST::name).toString());
        query.addBindValue(media_->parent()->data.at(ARTIST::isFavorite).toInt());
    
        int old_artist_playcount = media_->parent()->data.at(ARTIST::playcount).toInt();

        query.addBindValue(QString::number(old_artist_playcount));
        query.addBindValue( media_->parent()->data.at(ARTIST::rating).toFloat() );
    
        Debug::debug() << "database -> insert artist : " << query.exec();   

        QSqlQuery query2(command, *db.sqlDb());
        query2.next();
        artist_id = query2.value(0).toInt();	
      }
      else
      {
        artist_id = query.value(0).toInt();
      }
      Debug::debug() << "database -> selected artist_id " <<artist_id;
    }


    int album_id;
    //! ALBUM part in database
    {
      data.setValue(media_->data.at(ALBUM::name).toString());
      QString album_name_sql = db.sqlDb()->driver()->formatValue(data,false);

      QSqlQuery query("SELECT `id` FROM `albums` WHERE `name`="+album_name_sql+" AND `artist_id`="+QString::number(artist_id)+";", *db.sqlDb());
      Debug::debug() << "database -> #1 select album/artist : " << query.next();      

      
      if (!query.isValid())
      {
          command = QString("INSERT INTO albums(name,artist_id,cover,year,favorite,playcount,rating) VALUES(?,'%1',?,?,?,?,?);").arg(
            QString::number(artist_id) );

          QSqlQuery query(*db.sqlDb());
          query.prepare(command);
          query.addBindValue( media_->data.at(ALBUM::name).toString() );
          query.addBindValue( new_cover_name );
          query.addBindValue( QString::number(media_->data.at(ALBUM::year).toUInt() ) );
          query.addBindValue( media_->data.at(ALBUM::isFavorite).toInt() );
          query.addBindValue( QString::number(media_->data.at(ALBUM::playcount).toInt()) );      
          query.addBindValue( media_->data.at(ALBUM::rating).toFloat() );      
      
          Debug::debug() << "database -> insert album : " << query.exec();

          QSqlQuery query2("SELECT `id` FROM `albums` WHERE `name`="+album_name_sql+" AND " \
                           " `artist_id`="+QString::number(artist_id)+";", *db.sqlDb());

          query2.next(); 
          album_id = query2.value(0).toInt();
          Debug::debug() << "database -> insert album album_id: " << album_id;
      }
      else
      {
          album_id = query.value(0).toInt();
          command = QString("UPDATE albums  "   \
                      " SET name=?,         "   \
                      "     artist_id='%1', "   \
                      "     cover=?,        "   \
                      "     year=?,         "   \
                      "     playcount=?,    "   \
                      "     rating=?        "   \
                      "WHERE id='%2';").arg(
            QString::number(artist_id),
            QString::number(album_id) );

          QSqlQuery query(*db.sqlDb());
          query.prepare(command);
          query.addBindValue( media_->data.at(ALBUM::name).toString() );
          query.addBindValue( new_cover_name );
          query.addBindValue( QString::number(media_->data.at(ALBUM::year).toUInt()) );
          query.addBindValue( QString::number(media_->data.at(ALBUM::playcount).toInt()) );
          query.addBindValue( media_->data.at(ALBUM::rating).toFloat() );

          Debug::debug() << "database -> update album : " << query.exec();
      }

      Debug::debug() << "database -> selected album_id " << album_id;      
    } 
    
    //! TRACK part in database
    foreach ( MediaItemPtr track, media_->childItems)
    {    
        int trackId = track->data.at(TRACK::id).toInt();
            
        command = QString("UPDATE tracks "       \
                          "SET artist_id=%1, "   \
                          "    album_id=%2,  "   \
                          "    year_id=(SELECT id FROM years WHERE year='%3'), "   \
                          "    genre_id=(SELECT id FROM genres WHERE genre='%4') " \
                          " WHERE id=%5;").arg(
           QString::number(artist_id),
           QString::number(album_id),
           QString::number(media_->data.at(ALBUM::year).toUInt()),
           ui_album_genre->text(),
           QString::number(trackId) );

        QSqlQuery query(command, *db.sqlDb());
        Debug::debug() << "database -> update tracks id  " << trackId << " : " << query.exec();
    }

    //! Just once , If old cover exist -> take it !!
    recupCoverArt(new_cover_name,old_cover_name);
  
    //! Database Clean
    QSqlQuery queryClean1("DELETE FROM `albums` WHERE `id` NOT IN (SELECT `album_id` FROM `tracks` GROUP BY `album_id`);", *db.sqlDb());
    QSqlQuery queryClean2("DELETE FROM `genres` WHERE `id` NOT IN (SELECT `genre_id` FROM `tracks` GROUP BY `genre_id`);", *db.sqlDb());
    QSqlQuery queryClean3("DELETE FROM `artists` WHERE `id` NOT IN (SELECT `artist_id` FROM `tracks` GROUP BY `artist_id`);", *db.sqlDb());
    QSqlQuery queryClean4("DELETE FROM `years` WHERE `id` NOT IN (SELECT `year_id` FROM `tracks` GROUP BY `year_id`);", *db.sqlDb());

    QSqlQuery("COMMIT TRANSACTION;",*db.sqlDb());  
}


/*******************************************************************************
    do_changes_track 
*******************************************************************************/
void MediaItem_Edit_Dialog::do_changes_track()
{
    //! evaluate change
    if( media_->data.at(TRACK::title).toString() == ui_track_title->text() && 
        media_->data.at(TRACK::album).toString() == ui_album_name->text() &&
        media_->data.at(TRACK::artist).toString() == ui_artist_name->text() && 
        media_->data.at(TRACK::genre).toString() == ui_track_genre->text() &&
        QDate(media_->data.at(TRACK::year).toInt(),1,1) == ui_track_year->date() &&
        media_->data.at(TRACK::playcount).toInt() == ui_playcount->value() &&
        media_->data.at(TRACK::rating).toFloat() == ui_rating->rating() 
    )
    {
      Debug::debug() << "no change done --> close";
      isChanged_ = false;
      this->close();
      return;      
    }
    
    Debug::debug() << "change done --> apply";
    isChanged_ = true;
    
    //! apply new tag values    
    media_->data[TRACK::title]      =  ui_track_title->text();
    media_->data[TRACK::album]      =  ui_album_name->text();
    media_->data[TRACK::artist]     =  ui_artist_name->text();
    media_->data[TRACK::genre]      =  ui_track_genre->text();
    media_->data[TRACK::year]       =  ui_track_year->date().year();
    media_->data[TRACK::playcount]  =  ui_playcount->value();
    media_->data[TRACK::rating]     =  ui_rating->rating();
  
    MediaItemPtr album_media   = media_->parent();
    MediaItemPtr artist_media  = album_media->parent();
    
    //! apply to database
    QString command;
    Database db;
    if (!db.connect("MainThread")) return;
    QSqlQuery("BEGIN TRANSACTION;",*db.sqlDb());
    
    
    //! GENRE part in database
    {
        command = QString("SELECT id FROM genres WHERE genre='%1';").arg(
          media_->data.at(TRACK::genre).toString() );

        QSqlQuery query(command, *db.sqlDb());
        Debug::debug() << "database -> select genre : " << query.next();      

        if (!query.isValid())
        {
          command = QString("INSERT INTO genres (genre) VALUES('%1');").arg(
            media_->data.at(TRACK::genre).toString() );
  
          QSqlQuery query(command, *db.sqlDb());
          Debug::debug() << "database -> insert genre : " << query.exec();      
        }
    }
    
    
    //! YEAR part in database
    {
        command = QString("SELECT id FROM years WHERE year='%1';").arg(
          QString::number(media_->data.at(ALBUM::year).toUInt()) );      
          
        QSqlQuery query(command, *db.sqlDb());
        Debug::debug() << "database -> select year " << query.next();

        if (!query.isValid())
        {
          QSqlQuery query("INSERT INTO years (year) VALUES("+QString::number(media_->data.at(ALBUM::year).toUInt())+");", *db.sqlDb());
          Debug::debug() << "database -> insert year : " << query.exec();      
        }
    }
    
    
    int artist_id;
    //! ARTIST part in database
    {
      command = QString("SELECT id FROM artists WHERE name='%1';").arg(
        media_->data.at(TRACK::artist).toString() );
      
      QSqlQuery query(command, *db.sqlDb());
      Debug::debug() << "database -> select artist " << media_->data.at(TRACK::artist).toString() << ": " << query.next();
      
      if (!query.isValid()) 
      {
        QSqlQuery query(*db.sqlDb());
        query.prepare("INSERT INTO artists (name,favorite,rating) VALUES(?,?,?);");
        query.addBindValue( media_->data.at(TRACK::artist).toString() );
        query.addBindValue( artist_media->data.at(ARTIST::isFavorite).toInt() );
        query.addBindValue( artist_media->data.at(ARTIST::rating).toFloat() );
    
        Debug::debug() << "database -> insert artist : " << query.exec();
        
        QSqlQuery query2(command, *db.sqlDb());
        query2.next();
        artist_id = query2.value(0).toInt();
      }
      else
      {
        artist_id = query.value(0).toInt();
      }
      
      Debug::debug() << "database -> selected artist_id " << artist_id;
    }
    
    
    int album_id;
    const QString  new_cover_name  = MEDIA::getCoverName(media_->data.at(TRACK::artist).toString(),media_->data.at(TRACK::album).toString() );
    //! ALBUM part in database
    {
      command = QString("SELECT id FROM albums WHERE name='%1' AND artist_id='%2';").arg(
        media_->data.at(TRACK::album).toString(),
        QString::number(artist_id));
      
      QSqlQuery query(command, *db.sqlDb());
      Debug::debug() << "database -> select album/artist : " << query.next();      

      
      if (!query.isValid()) 
      {
          command = QString("INSERT INTO albums(name,artist_id,cover,year,favorite,playcount,rating) VALUES(?,'%1',?,?,?,?,?);").arg(
            QString::number(artist_id) );

          QSqlQuery query(*db.sqlDb());
          query.prepare(command);
          query.addBindValue( media_->data.at(TRACK::album).toString() );
          query.addBindValue( new_cover_name );
          query.addBindValue( QString::number(media_->data.at(TRACK::year).toUInt() ) );
          query.addBindValue( album_media->data.at(ALBUM::isFavorite).toInt() );
          query.addBindValue( QString::number(album_media->data.at(ALBUM::playcount).toInt()) );
          query.addBindValue( album_media->data.at(ALBUM::rating).toFloat() );
      
          Debug::debug() << "database -> insert album : " << query.exec();

          command = QString("SELECT id FROM albums WHERE name='%1' AND artist_id='%2';").arg(
             media_->data.at(TRACK::album).toString(),
             QString::number(artist_id));
      
          QSqlQuery query2(command, *db.sqlDb());
          query2.next(); 
          album_id = query2.value(0).toInt();  
      }
      else 
      {
          album_id = query.value(0).toInt();
          command = QString("UPDATE albums     "   \
                      " SET name=?,            "   \
                      "     artist_id='%1',    "   \
                      "     cover=?,           "   \
                      "     year=?,            "   \
                      "     rating=? WHERE id='%2';").arg(
            QString::number(artist_id),
            QString::number(album_id) );

          QSqlQuery query(*db.sqlDb());
          query.prepare(command);
          query.addBindValue( media_->data.at(TRACK::album).toString() );
          query.addBindValue( new_cover_name );
          query.addBindValue( QString::number(media_->data.at(TRACK::year).toUInt()) );
          query.addBindValue( album_media->data.at(ALBUM::rating).toFloat());

          Debug::debug() << "database -> update album : " << query.exec();
      }
      
      Debug::debug() << "database -> selected album_id " <<album_id;      
    }     
    
    
    //! TRACK part in database
    {    
        int trackId = media_->data.at(TRACK::id).toInt();
            
        command = QString("UPDATE tracks "       \
                          "SET artist_id=%1, "   \
                          "    album_id=%2,  "   \
                          "    year_id=(SELECT id FROM years WHERE year='%3'),    "  \
                          "    genre_id=(SELECT id FROM genres WHERE genre='%4'), "  \
                          "    rating=?      "   \
                          " WHERE id=%5;").arg(
           QString::number(artist_id),
           QString::number(album_id),
           QString::number(media_->data.at(TRACK::year).toUInt()),
           media_->data.at(TRACK::genre).toString(),
           QString::number(trackId) );

        QSqlQuery query(command, *db.sqlDb());
        query.addBindValue( media_->data.at(TRACK::rating).toFloat());
        Debug::debug() << "database -> update tracks id  " << trackId << " : " << query.exec();
    }

    //! Database clean
    QSqlQuery queryClean1("DELETE FROM `albums` WHERE `id` NOT IN (SELECT `album_id` FROM `tracks` GROUP BY `album_id`);", *db.sqlDb());
    QSqlQuery queryClean2("DELETE FROM `genres` WHERE `id` NOT IN (SELECT `genre_id` FROM `tracks` GROUP BY `genre_id`);", *db.sqlDb());
    QSqlQuery queryClean3("DELETE FROM `artists` WHERE `id` NOT IN (SELECT `artist_id` FROM `tracks` GROUP BY `artist_id`);", *db.sqlDb());
    QSqlQuery queryClean4("DELETE FROM `years` WHERE `id` NOT IN (SELECT `year_id` FROM `tracks` GROUP BY `year_id`);", *db.sqlDb());

    QSqlQuery("COMMIT TRANSACTION;",*db.sqlDb());    
}

