/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "networkaccess.h"
#include "constants.h"
#include "debug.h"


static const QString USER_AGENT = APP_NAME + " " + VERSION;

/*
********************************************************************************
*                                                                              *
*    Class NetworkReply                                                        *
*                                                                              *
********************************************************************************
*/
NetworkReply::NetworkReply(QNetworkReply *networkReply) : QObject(networkReply) 
{
    this->networkReply = networkReply;
    followRedirects = true;

    timer = new QTimer(this);
    timer->setInterval(60000);
    timer->setSingleShot(true);
    connect(timer, SIGNAL(timeout()), SLOT(abort()));
    timer->start();
}

void NetworkReply::finished() 
{
    timer->stop();

    if (networkReply->error() != QNetworkReply::NoError) {
        Debug::debug() << networkReply->url() << "finished with error" << networkReply->error();
        networkReply->deleteLater();
        return;
    }

    if (followRedirects) {
        QUrl redirection = networkReply->attribute(QNetworkRequest::RedirectionTargetAttribute).toUrl();
        if (redirection.isValid()) {

            QNetworkReply *redirectReply = HTTP()->simpleGet(redirection, networkReply->operation());

            setParent(redirectReply);
            networkReply->deleteLater();
            networkReply = redirectReply;

            // when the request is finished we'll invoke the target method
            connect(networkReply, SIGNAL(finished()), this, SLOT(finished()), Qt::QueuedConnection);

            return;
        }
    }

    emit finished(networkReply);

    // get the HTTP response body
    QByteArray bytes = networkReply->readAll();
    emit data(bytes);

    // bye bye my reply
    // this will also delete this NetworkReply as the QNetworkReply is its parent
    networkReply->deleteLater();
}

void NetworkReply::requestError(QNetworkReply::NetworkError /*code*/)
{
    timer->stop();
    emit error(networkReply);
}

void NetworkReply::abort()
{
    Debug::debug() << "HTTP timeout" << networkReply->url();
    networkReply->abort();
    emit error(networkReply);
}

/*
********************************************************************************
*                                                                              *
*    Class NetworkAccess                                                       *
*                                                                              *
********************************************************************************
*/ 
NetworkAccess::NetworkAccess(QObject* parent) : QObject( parent ) {}

QNetworkReply* NetworkAccess::simpleGet(QUrl url, int operation)
{
    QNetworkAccessManager *manager = new QNetworkAccessManager(this);

    QNetworkRequest request(url);
    request.setRawHeader("User-Agent", USER_AGENT.toUtf8());
    request.setRawHeader("Connection", "Keep-Alive");
      
    
    QNetworkReply *networkReply;
    switch (operation) {

    case QNetworkAccessManager::GetOperation:
        Debug::debug() << "GET" << url.toEncoded();
        networkReply = manager->get(request);
        break;

    case QNetworkAccessManager::HeadOperation:
        Debug::debug() << "HEAD" << url.toEncoded();
        networkReply = manager->head(request);
        break;

    default:
        Debug::debug() << "Unknown operation:" << operation;
        return 0;

    }

    // error handling
    connect(networkReply, SIGNAL(error(QNetworkReply::NetworkError)),
            this, SLOT(error(QNetworkReply::NetworkError)), Qt::QueuedConnection);

    return networkReply;

}

NetworkReply* NetworkAccess::get(const QUrl url)
{
    QNetworkReply *networkReply = simpleGet(url);
    NetworkReply *reply = new NetworkReply(networkReply);

    // error signal
    connect(networkReply, SIGNAL(error(QNetworkReply::NetworkError)),
            reply, SLOT(requestError(QNetworkReply::NetworkError)), Qt::QueuedConnection);

    // when the request is finished we'll invoke the target method
    connect(networkReply, SIGNAL(finished()), reply, SLOT(finished()), Qt::QueuedConnection);

    return reply;

}

NetworkReply* NetworkAccess::head(const QUrl url)
{
    QNetworkReply *networkReply = simpleGet(url, QNetworkAccessManager::HeadOperation);
    NetworkReply *reply = new NetworkReply(networkReply);
    reply->followRedirects = false;

    // error signal
    connect(networkReply, SIGNAL(error(QNetworkReply::NetworkError)),reply, SLOT(requestError(QNetworkReply::NetworkError)), Qt::QueuedConnection);

    // when the request is finished we'll invoke the target method
    connect(networkReply, SIGNAL(finished()), reply, SLOT(finished()), Qt::QueuedConnection);

    return reply;

}

QNetworkReply* NetworkAccess::syncGet(QUrl url)
{
    working = true;

    networkReply = simpleGet(url);
    connect(networkReply, SIGNAL(metaDataChanged()),this, SLOT(syncMetaDataChanged()), Qt::QueuedConnection);
    connect(networkReply, SIGNAL(finished()),this, SLOT(syncFinished()), Qt::QueuedConnection);
    connect(networkReply, SIGNAL(error(QNetworkReply::NetworkError)),this, SLOT(error(QNetworkReply::NetworkError)), Qt::QueuedConnection);

    // A little trick to make this function blocking
    while (working) {
        // Do something else, maybe even network processing events
        qApp->processEvents();
    }

    networkReply->deleteLater();
    return networkReply;

}

QNetworkReply* NetworkAccess::syncHead(QUrl url)
{
    working = true;

    networkReply = simpleGet(url, QNetworkAccessManager::HeadOperation);
    connect(networkReply, SIGNAL(finished()),this, SLOT(syncFinished()), Qt::QueuedConnection);
    connect(networkReply, SIGNAL(error(QNetworkReply::NetworkError)),this, SLOT(error(QNetworkReply::NetworkError)), Qt::QueuedConnection);

    // A little trick to make this function blocking
    while (working) {
        // Do something else, maybe even network processing events
        qApp->processEvents();
    }

    networkReply->deleteLater();
    return networkReply;

}

void NetworkAccess::syncMetaDataChanged()
{
    QUrl redirection = networkReply->attribute(QNetworkRequest::RedirectionTargetAttribute).toUrl();
    if (redirection.isValid())
    {
        Debug::debug() << "Redirect" << redirection;
        networkReply->deleteLater();
        syncGet(redirection);
    }

}

void NetworkAccess::syncFinished()
{
    //Debug::debug() << "syncFinished";
    // got it!
    working = false;
}

void NetworkAccess::error(QNetworkReply::NetworkError code)
{
    //! get the QNetworkReply that sent the signal
    QNetworkReply *networkReply = static_cast<QNetworkReply *>(sender());
    if (!networkReply) {
        Debug::debug() << "Cannot get sender";
        return;
    }

    Debug::debug() <<"NetworkAccess" <<  networkReply->errorString() << code;

    networkReply->deleteLater();
}

QByteArray NetworkAccess::syncGetBytes(QUrl url)
{
    return syncGet(url)->readAll();
}

QString NetworkAccess::syncGetString(QUrl url)
{
    return QString::fromUtf8(syncGetBytes(url));
}

bool NetworkAccess::syncTest(QUrl url)
{
    return (syncHead(url)->error() == QNetworkReply::NoError);
}



/*
********************************************************************************
*                                                                              *
*    Class NetworkManager                                                      *
*                                                                              *
********************************************************************************
*/ 
NetworkManager* NetworkManager::INSTANCE = 0;

NetworkManager::NetworkManager() 
{
    INSTANCE = this;  
}


NetworkAccess* NetworkManager::http()
{
  if (m_http.contains(QThread::currentThread())) 
  {
    return m_http.value(QThread::currentThread());
  } 
  else 
  {
    // Debug::debug() << "NetworkAccess for thread" << QThread::currentThread();
    NetworkAccess *http = new NetworkAccess();
    m_http.insert(QThread::currentThread(), http);
    return http;
  }  
}
