/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "threadmanager.h"
#include "CollectionModel/collectionpopulator.h"
#include "CollectionModel/plsfilepopulator.h"
#include "Database/databasebuilder.h"

#include "Playlist/playlistmodel.h"
#include "Playlist/playlistpopulator.h"
#include "Playlist/playlistwriter.h"
#include "Playlist/playlistdbwriter.h"

#include "AlbumCover/coverthread.h"

#include "Widget/statuswidget.h"   

#include "debug.h" 

#include <QSignalMapper>

ThreadManager* ThreadManager::INSTANCE = 0;
/*
********************************************************************************
*                                                                              *
*    Class ThreadManager                                                       *
*                                                                              *
********************************************************************************
*/                   
ThreadManager::ThreadManager()
{
    INSTANCE = this;
    _threadPool          = new QThreadPool(this);
    
    // Qrunnable
    _playlistPopulator   = new PlaylistPopulator();
    _playlistPopulator->setAutoDelete(false);

    _playlistWriter      = new PlaylistWriter();
    _playlistWriter->setAutoDelete(false);    

    _playlistDbWriter    = new PlaylistDbWriter();
    _playlistDbWriter->setAutoDelete(false);    

    // QThread 
    _databaseBuilder     = new DataBaseBuilder();
    _collectionPopulator = new CollectionPopulator();
    _plsFilePopulator    = new PlsFilePopulator();
    _coverThread         = 0;
        
    // connection
    QObject::connect(_playlistPopulator,SIGNAL(playlistPopulated()),this,SIGNAL(playlistPopulated()));
    QObject::connect(_playlistPopulator,SIGNAL(async_load(QString,int)),this,SLOT(slot_load_async(QString,int)));

    QObject::connect(_playlistWriter,SIGNAL(playlistSaved()),this,SIGNAL(playlistSaved()));
    QObject::connect(_playlistDbWriter,SIGNAL(playlistSaved()),this,SIGNAL(playlistSaved()));
    
    QObject::connect(_databaseBuilder,SIGNAL(buildingFinished()),this,SLOT(dbBuildFinish()));
    QObject::connect(_databaseBuilder,SIGNAL(buildingProgress(int)),this,SLOT(dbBuildProgressChanged(int)));

    QObject::connect(_collectionPopulator,SIGNAL(populatingFinished()),this,SLOT(modelCollectionPopulated()));
    QObject::connect(_collectionPopulator,SIGNAL(populatingProgress(int)),this,SLOT(modelCollectionPopulatingChanged(int)));
    QObject::connect(_collectionPopulator, SIGNAL(populatingRunning()), this,SLOT(modelCollectionPopulatingRunning()));
 
    QObject::connect(_plsFilePopulator,SIGNAL(populatingFinished()),this,SLOT(modelPlaylistPopulated()));
    QObject::connect(_plsFilePopulator,SIGNAL(populatingProgress(int)),this,SLOT(modelPlaylistPopulatingChanged(int)));
}

ThreadManager::~ThreadManager()
{
    Debug::debug() << "ThreadManager -> wait to finish ..."; 
    _threadPool->waitForDone();
    delete _playlistPopulator;
    delete _playlistWriter;
    delete _playlistDbWriter;
    delete _databaseBuilder;
    delete _collectionPopulator;
    delete _plsFilePopulator;
    delete _threadPool;
}



void ThreadManager::stopThread() 
{
    Debug::debug() << "ThreadManager -> stop running thread"; 
    if(_databaseBuilder->isRunning())
      cancelThread(DB_THREAD);
    
    if(_collectionPopulator->isRunning())
      cancelThread(POPULATOR_C_THREAD);

    if(_plsFilePopulator->isRunning())
      cancelThread(POPULATOR_P_THREAD);
    
    if(_coverThread != 0)
      if(_coverThread->isRunning())
        cancelThread(COVER_SEARCH_THREAD);    
}

/*******************************************************************************
 setModel
*******************************************************************************/
void ThreadManager::setModel(PlaylistModel* model)
{
    m_model = model;
    _playlistPopulator->setModel(m_model);
    _playlistWriter->setModel(m_model);
    _playlistDbWriter->setModel(m_model); 
}

/*******************************************************************************
  slot_load_async
*******************************************************************************/
void ThreadManager::slot_load_async(QString url,int row)
{
    AsynchronousLoadTask* loader = new AsynchronousLoadTask(url, row);
    loader->setModel(m_model);
    loader->start_asynchronous_download();
}


/*******************************************************************************
 PlayQueue Population 
*******************************************************************************/
void ThreadManager::playlistAddFiles(const QStringList &files) 
{
    if(_playlistPopulator->isRunning()) return;
    _playlistPopulator->addFiles(files);
    _threadPool->start(_playlistPopulator);
}

void ThreadManager::playlistAddFile(const QString &file) 
{
    if(_playlistPopulator->isRunning()) return;
    _playlistPopulator->addFile(file);
    _threadPool->start (_playlistPopulator);
}

void ThreadManager::playlistAddUrls(QList<QUrl> listUrl, int playlist_row) 
{
    if(_playlistPopulator->isRunning()) return;
    _playlistPopulator->addUrls(listUrl, playlist_row);
    _threadPool->start (_playlistPopulator);
}

void ThreadManager::playlistAddMediaItems(QList<MediaItemPtr> list, int playlist_row) 
{
    if(_playlistPopulator->isRunning()) return;
    _playlistPopulator->addMediaItems(list, playlist_row);
    _threadPool->start (_playlistPopulator);
}


/*******************************************************************************
 PlayQueue to playlist Writer 
*******************************************************************************/
void ThreadManager::playlistSaveToFile(const QString &filename)
{
    if(_playlistWriter->isRunning()) return;
    _playlistWriter->saveToFile(filename);
    _threadPool->start (_playlistWriter);
}

void ThreadManager::playlistSaveToDb(const QString &name, int bd_id)
{
    if(_playlistDbWriter->isRunning()) return;
    _playlistDbWriter->saveToDatabase(name, bd_id);
    _threadPool->start (_playlistDbWriter);
}

/*******************************************************************************
 Database Scanner Thread 
*******************************************************************************/
void ThreadManager::databaseBuild(QStringList listDir)
{
    if(_databaseBuilder->isRunning())
      cancelThread(DB_THREAD);      
    
    _databaseBuilder->rebuildFolder(listDir);
    _databaseBuilder->start();
    
    uint i = StatusWidget::instance()->startProgressMessage(tr("Updating Music Collection") + " (0%)");
    messageIds.insert("DbUpdate", i);
}

void ThreadManager::dbBuildProgressChanged(int progress)
{
    QString message = QString(tr("Updating Music Collection") + " (%1%)").arg(QString::number(progress));
    StatusWidget::instance()->updateProgressMessage( messageIds.value("DbUpdate"), message );
}

void ThreadManager::dbBuildFinish()
{
    if (messageIds.contains("DbUpdate"))
      StatusWidget::instance()->stopProgressMessage( messageIds.take("DbUpdate") );
    
    emit dbBuildFinished();
}

bool ThreadManager::isDbRunning()
{
    return _databaseBuilder->isRunning();
}



/*******************************************************************************
 Collection Model Population thread 
*******************************************************************************/
void ThreadManager::populateCollectionModel()
{
   Debug::debug() << "ThreadManager -> start a new collection populator"; 
   if(_collectionPopulator->isRunning()) {
     cancelThread(POPULATOR_C_THREAD);
   }
    
   _collectionPopulator->start();
   
   uint i = StatusWidget::instance()->startProgressMessage(tr("Loading Music Collection"));
   messageIds.insert("LoadMusic", i);
}

void ThreadManager::modelCollectionPopulatingChanged(int progress)
{
    QString message = QString(tr("Loading Music Collection") + " (%1%)").arg(QString::number(progress));
    StatusWidget::instance()->updateProgressMessage( messageIds.value("LoadMusic"), message );
}

void ThreadManager::modelCollectionPopulated()
{
    if (messageIds.contains("LoadMusic"))
      StatusWidget::instance()->stopProgressMessage( messageIds.take("LoadMusic") );
    
    emit modelPopulationFinished(MODEL_COLLECTION);
}

void ThreadManager::modelCollectionPopulatingRunning()
{
    // in order to have graphics Scene updated every 5 seconds
    emit modelPopulationFinished(MODEL_COLLECTION);
}



/*******************************************************************************
 Playlist Model Population thread 
*******************************************************************************/
void ThreadManager::populatePlsFileModel()
{
   Debug::debug() << "ThreadManager -> start a new playlist populator"; 
   if(_plsFilePopulator->isRunning())
     cancelThread(POPULATOR_P_THREAD);
    
   _plsFilePopulator->start();
   
   uint i = StatusWidget::instance()->startProgressMessage(tr("Loading Playlist File"));
   messageIds.insert("LoadPlaylist", i);   
}

void ThreadManager::modelPlaylistPopulatingChanged(int progress)
{
    QString message = QString(tr("Loading Playlist File") + " (%1%)").arg(QString::number(progress));
    StatusWidget::instance()->updateProgressMessage( messageIds.value("LoadPlaylist"), message );
}

void ThreadManager::modelPlaylistPopulated()
{
    if (messageIds.contains("LoadPlaylist"))
      StatusWidget::instance()->stopProgressMessage( messageIds.take("LoadPlaylist") );
    
    emit modelPopulationFinished(MODEL_PLAYLIST);
}

/*******************************************************************************
  Cover Search Thread 
*******************************************************************************/
void ThreadManager::startCoverSearch()
{
    if(!_coverThread) {
      _coverThread = new CoverThread();
      
      connect(_coverThread,SIGNAL(finished()),this,SLOT(coverSearchFinished()));
      connect(_coverThread,SIGNAL(progress(int)),this,SLOT(coverSearchProgress(int)));
    }
  
    if(_coverThread->isRunning()) 
      cancelThread(COVER_SEARCH_THREAD);
    
    _coverThread->start();

    uint i = StatusWidget::instance()->startProgressMessage(tr("Fetching Album Cover"));
    messageIds.insert("CoverUpdate", i);
}

void ThreadManager::startCoverSearch(const QString& artist, const QString& album)
{
    if(!_coverThread) {
      _coverThread = new CoverThread();
      
      connect(_coverThread,SIGNAL(finished()),this,SLOT(coverSearchFinished()));
      connect(_coverThread,SIGNAL(progress(int)),this,SLOT(coverSearchProgress(int)));
    }
  
    if(_coverThread->isRunning()) 
      cancelThread(COVER_SEARCH_THREAD);
    
    _coverThread->setRequest(artist, album);    
    _coverThread->start();
}


void ThreadManager::coverSearchFinished()
{
    if (messageIds.contains("CoverUpdate"))
      StatusWidget::instance()->stopProgressMessage( messageIds.take("CoverUpdate") );  
}

void ThreadManager::coverSearchProgress(int progress)
{
    QString message = QString(tr("Fetching Album Cover") + " (%1%)").arg(QString::number(progress));
    StatusWidget::instance()->updateProgressMessage( messageIds.value("CoverUpdate"), message );  
}




/*******************************************************************************
  Cancel Thread 
*******************************************************************************/
void ThreadManager::cancelThread(E_THREAD thread)
{
    switch(thread) {
      case DB_THREAD:
         Debug::warning() << "ThreadManager ->  database builder isRunning : Stop it !!"; 
        _databaseBuilder->setExit(true);
        _databaseBuilder->wait();      // wait end of thread should be fast
        _databaseBuilder->setExit(false);
        if (messageIds.contains("DbUpdate"))
          StatusWidget::instance()->stopProgressMessage( messageIds.take("DbUpdate") );

       break;
       
      case POPULATOR_C_THREAD:
         Debug::warning() << "ThreadManager ->  collection populator isRunning : Stop it !!"; 
        _collectionPopulator->setExit(true);
        _collectionPopulator->wait();      // wait end of thread should be fast
        _collectionPopulator->setExit(false);
        if (messageIds.contains("LoadMusic"))
          StatusWidget::instance()->stopProgressMessage( messageIds.take("LoadMusic") );

       break;
      
      case  POPULATOR_P_THREAD:
         Debug::warning() << "ThreadManager ->  playlist populator isRunning : Stop it !!"; 
        _plsFilePopulator->setExit(true);
        _plsFilePopulator->wait();      
        _plsFilePopulator->setExit(false);               

        if (messageIds.contains("LoadPlaylist"))
          StatusWidget::instance()->stopProgressMessage( messageIds.take("LoadPlaylist") );	

      break;
      
      case COVER_SEARCH_THREAD:
        _coverThread->setExit(true);
        _coverThread->wait();      
        _coverThread->setExit(false);      
        if (messageIds.contains("CoverUpdate"))
          StatusWidget::instance()->stopProgressMessage( messageIds.take("CoverUpdate") );	

      break;
      
      default:break;
    }
}


